#if defined uint_included
	#endinput
#endif
#define uint_included

/*

As soon as the value exceeds int limit (2147483647), SourcePawn starts to have problems, example:

int x = 2147483650 / 2; // Must be: 1073741825
PrintToServer("%%u = %u\n%%d = %d\n%u > 1000 = %s", x, x, x, x > 1000 ? "yes" : "no");

// Result: all values are incorrect + comparison does not work:
%u = 3221225473
%d = -1073741823
3221225473 > 1000 = no

*/

#define UINT_SIZE 12

/**
 * Math operations with unsigned int ('+', '-', '*', '/', '%')
 * 
 * Example:
 * int i = uint(2147483650, '/', 2);
 * PrintToServer("%u", i); // 1073741825
 * 
 * Note:
 * unsigned int limit (4294967295) is not violated:
 * 4294967295 + 1 = 4294967295
 * 0 - 1 = 0
 * 4294967295 * 2 = 4294967295
 */
native int uint(int a, int op, int b);

/**
 * Comparison two unsigned int values
 * 
 * Example:
 * bool b = uintcmp(5, 3) >= 0; // true, if 5 >= 3 (instead of operator >=, use any. 0 must be always).
 */
native int uintcmp(int a, int b);

/**
 * Replacement FormatTime, because:
 * 
 * char s[128];
 * FormatTime(s, sizeof(s), "%d.%m.%Y - %H:%M", 2147483648);
 * [SM] Exception reported: Invalid time format or buffer too small (or I'm stupid and forgot to convert stamp in unsigned int)
 */
native bool FormatUTime(char[] buffer, int maxlength, const char[] format, int stamp=0);

// Convert unsigned int to string (IntToString can't) (return: Number of cells written)
stock int UIntToString(int value, char[] s, int size) {
    return FormatEx(s, size, "%u", value);
}

// Extract unsigned int from the field (SQL_FetchInt can't)
stock int SQL_FetchUInt(Handle results, int field, DBResult &result=DBVal_Error, int error=0)
{
    char s[UINT_SIZE];
    return SQL_FetchString(results, field, s, sizeof(s), result) && result == DBVal_Data ? StringToInt(s) : error;
}

public Extension __ext_uint = 
{
	name = "uint",
	file = "uint.ext",
#if defined AUTOLOAD_EXTENSIONS
	autoload = 1,
#else
	autoload = 0,
#endif
#if defined REQUIRE_EXTENSIONS
	required = 1,
#else
	required = 0,
#endif
};

#if !defined REQUIRE_EXTENSIONS
public void __ext_uint_SetNTVOptional()
{
	MarkNativeAsOptional("uint");
	MarkNativeAsOptional("uintcmp");
	MarkNativeAsOptional("FormatUTime");
}
#endif